import {
	btn,
	comp,
	Component,
	ContainerFieldValue,
	DateInterval, DateTime,
	DisplayField,
	displayfield, EntityID,
	fieldset,
	Form,
	form,
	Format, h3,
	t, tbar
} from "@intermesh/goui";
import {Chart, client, jmapds, modules, router} from "@intermesh/groupoffice-core";
import {InvoiceWindow} from "./InvoiceWindow.js";

export class ProjectFinance extends Component {
	private form: Form<ContainerFieldValue>;
	private projectId?: EntityID;
	private hoursChart?: Chart<any, number[], string>;
	private profitChart?: Chart<any, number[], string>;
	private billableChart?: Chart<any, number[], string>;
	private taskChart?: Chart<any, number[], string>;
	constructor() {
		super();

		this.cls = "card project-finance";
		this.hidden = true;

		this.itemId = Component.uniqueID();


		jmapds("Registration").on("change", this.onChange, {bind: this});
		jmapds("FinanceDocument").on("change", this.onChange, {bind: this});
		this.on("remove", () => {
			jmapds("Registration").un("change", this.onChange);
			jmapds("FinanceDocument").un("change", this.onChange);
		})


		this.items.add(
			this.form = form({
					cls: "label-align-left",
					listeners: {
						beforesetvalue: (field, e) => {

							// calculate some extra form values before setting the form
							e.value.billedMinutes = e.value.billedRegistrations.reduce((c:number, v:any) => c + v.minutes, 0);
							e.value.billedFee = e.value.billedRegistrations.reduce((c:number, v:any) => c + v.fee, 0);
							e.value.billedFeeCosts = e.value.billedRegistrations.reduce((c:number, v:any) => c + v.costs, 0);


							if (!e.value.quotedMinutes) {
								e.value.quotedMinutes = e.value.budgetMinutes;
							}
							e.value.tasksTotal = e.value.tasksTodo + e.value.tasksCompleted;
							e.value.billableTotal = e.value.quoteAmount + e.value.billableFee;
							e.value.outstanding = e.value.quoteAmount + e.value.billableFee - e.value.invoicedAmount;
							e.value.profit = e.value.quoteAmount + e.value.billableFee - e.value.quoteCosts - e.value.quotedFeeCosts - e.value.billableFeeCosts;
							e.value.totalHours = e.value.quotedMinutes + e.value.billableMinutes;
							e.value.totalFeeCosts = e.value.billableFeeCosts + e.value.quotedFeeCosts;

							if(e.value.billableTotal === 0) {
								this.hide();
							} else {
								this.show();
							}

							this.drawBillableChart(e.value);
							this.drawProfitChart(e.value);
						}
					}
				},
				tbar({}, h3(t("Finance"))),

				comp({cls: "project-finance-graphs"},

					fieldset({legend: "Invoiced", cls: "vbox gap"},

						comp({tagName: "canvas", id: "project-finance-billable-chart-" + this.itemId}),

						displayfield({
							cls: "right",
							name: "quoteAmount",
							label: t("Quote"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						displayfield({
							cls: "right border-bottom",
							name: "billableFee",
							label: t("Billable fee"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						displayfield({
							cls: "right",
							name: "billableTotal",
							label: t("Total"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						displayfield({
							cls: "right border-bottom",
							name: "invoicedAmount",
							label: t("Invoiced"),
							renderer: (v) => {
								return Format.money(-v);
							}
						}),

						displayfield({
							cls: "right",
							name: "outstanding",
							label: t("Outstanding"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						comp({flex: 1}), // fill space till bottom in hbox layout

						...(modules.isAvailable("business", "registration") ? [btn({
							text: t("Create invoice"),
							cls: "outlined",
							handler: async () => {

								const fv = this.form.value;
								const win = new InvoiceWindow(fv);
								win.show();
							}
						})] : [])
					),

					fieldset({legend: "Profit", cls: "vbox gap"},

						comp({tagName: "canvas", id: "project-finance-profit-chart-" + this.itemId}),

						displayfield({
							cls: "right",
							name: "quoteAmount",
							label: t("Quote amount"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						displayfield({
							cls: "right",
							name: "quoteCosts",
							label: t("Quote costs"),
							renderer: (v) => {
								return Format.money(-v);
							}
						}),

						displayfield({
							cls: "right",
							name: "billableFee",
							label: t("Billable fee"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						displayfield({
							cls: "right border-bottom",
							name: "totalFeeCosts",
							label: t("Fee cost"),
							renderer: (v) => {
								return Format.money(-v);
							}
						}),

						displayfield({
							cls: "right",
							name: "profit",
							label: t("Profit"),
							renderer: (v) => {
								return Format.money(v);
							}
						}),

						...(modules.isAvailable("business", "registration") ? [
							comp({flex: 1}), // fill space till bottom in hbox layout
							btn({
							cls: "outlined",
							text: t("View time entries"),
							handler: () => {
								let url = "registration/list/" + this.projectId + "//billable/<" + (new DateTime()).addDays(1).format("Y-m-d");
								void router.goto(url);
							}

						})] : [])
					),
				)
			)
		)


	}



	private drawBillableChart(newValue:any) {

		const billableAmount = (newValue.quoteAmount + newValue.billableFee), data = {
			labels: [
				'Invoiced of quote',
				'Billed fee',
				'Invoicable for quote',
				'Billable fee',
			],
			centerLabel: billableAmount ? Math.ceil((100 - (newValue.outstanding / billableAmount) * 100)) + "%" : "-",
			datasets: [{
				data: [
					newValue.invoicedAmount - newValue.billedFee,
					newValue.billedFee,
					newValue.quoteAmount + newValue.billedFee - newValue.invoicedAmount,
					newValue.billableFee - newValue.billedFee
				],
				backgroundColor: [
					'rgb(54, 162, 235)',
					'rgb(54, 162, 235)',
					'rgb(255, 99, 132)',
					'rgb(255, 99, 132)'
				],
				hoverOffset: 4
			}]
		};

		if(!this.billableChart) {
			this.billableChart = new Chart("project-finance-billable-chart-" + this.itemId, {
				type: 'doughnut',
				data: data,
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false
						},
						//doughnutCenterLabel: {}
					}
				}
			});
		} else {
			this.billableChart.data = data;
			this.billableChart.update();
		}
	}

	private drawProfitChart(newValue:any) {

		console.log(newValue);

		const billableAmount = (newValue.quoteAmount + newValue.billableFee),
			margin = billableAmount ?
				Math.ceil((1 - (newValue.quoteCosts + newValue.quotedFeeCosts + newValue.billableFeeCosts) / billableAmount) * 100) + "%" : "-";

		const data = {
			labels: [
				t('Profit'),
				t('Billable fee'),
				t('Quote costs'),
				t('Fee costs'),
			],
			centerLabel: margin,
			datasets: [{
				data: [
					newValue.quoteAmount - newValue.quoteCosts - newValue.quotedFeeCosts,
					newValue.billableFee - newValue.billableFeeCosts,
					newValue.quoteCosts,
					newValue.quotedFeeCosts + newValue.billableFeeCosts
				],
				backgroundColor: [
					'rgb(54, 162, 235)',
					'rgb(54,235,63)',
					'rgb(255, 99, 132)',
					'rgb(255, 99, 132)',

				],
				hoverOffset: 4
			}]
		};

		if(!this.profitChart) {
			this.profitChart = new Chart("project-finance-profit-chart-" + this.itemId, {
				type: 'doughnut',
				data: data,
				options: {
					responsive: true,
					plugins: {
						legend: {
							display: false
						},
						//doughnutCenterLabel: {}
					}
				}
			});
		} else {
			this.profitChart.data = data;
			this.profitChart.update();
		}
	}

	private onChange() {
		if(this.projectId) {
			this.load(this.projectId);
		}
	}


	public async load(projectId:EntityID) {
		this.projectId = projectId;
		this.form.value  = await client.jmap("Project3/getFinanceData", {projectId: projectId});
	}
}